/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */ 

package ws.ladder.db.base;

import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;

import ws.ladder.db.common.Property;
import ws.ladder.db.option.OptionModel;

@Entity
@Inheritance(strategy = InheritanceType.JOINED)
public class Artifact extends Identity {

	private OptionModel optionModel;

	@OneToOne(mappedBy = "artifact", cascade = { CascadeType.ALL })
	public OptionModel getOptionModel() {
		return this.optionModel;
	}

	public void setOptionModel(OptionModel optionModel) {
		this.optionModel = optionModel;
	}

	private Set<Property> property = new HashSet<Property>();

	@OneToMany(mappedBy = "artifact", cascade = CascadeType.ALL, orphanRemoval = true)
	public Set<Property> getProperty() {
		return this.property;
	}

	public void setProperty(Set<Property> propertys) {
		this.property = propertys;
	}

	private boolean disabled;

	@Column
	public boolean isDisabled() {
		return disabled;
	}

	public void setDisabled(boolean disabled) {
		this.disabled = disabled;
	}

	private Person ownedBy;

	@ManyToOne
	public Person getOwnedBy() {
		return this.ownedBy;
	}

	public void setOwnedBy(Person ownedBy) {
		this.ownedBy = ownedBy;
	}	
	
	private ArtifactType artifactType;

	public void setArtifactType(ArtifactType value) {
		this.artifactType = value;
	}

	@ManyToOne(optional = false)
	public ArtifactType getArtifactType() {
		return this.artifactType;
	}
	
	private ArtifactState state;

	public void setState(ArtifactState value) {
		this.state = value;
	}

	@ManyToOne(optional = false)
	public ArtifactState getState() {
		return this.state;
	}	
}
