/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */ 

package ws.ladder.db.common;

import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;

import ws.ladder.annotation.DataModel;
import ws.ladder.db.base.Artifact;
import ws.ladder.db.base.Identity;
import ws.ladder.db.option.Option;

@Entity
@DataModel(userKeyProperty = "name", collectionUserKey = true)
public class Property extends Identity {

	private Artifact artifact;
	private Option option;
	private Set<PropertyBinding> propertyBindings = new HashSet<PropertyBinding>();

	private ValueType valueType;
	private Set<Value> values = new HashSet<Value>();

	private int maxOccurs = 1;
	private int minOccurs = 0;
	private int orderIndex = -1;

	private boolean confidential;
	private boolean encrypted;
	private boolean consumerReadOnly;
	private boolean consumerVisible;

	@ManyToOne(optional = true, fetch = FetchType.LAZY)
	public Artifact getArtifact() {
		return artifact;
	}

	public void setArtifact(Artifact artifact) {
		this.artifact = artifact;
	}

	@ManyToOne(optional = true, fetch = FetchType.LAZY)
	public Option getOption() {
		return option;
	}

	public void setOption(Option option) {
		this.option = option;
	}

	@OneToMany(mappedBy = "property", cascade = CascadeType.ALL, orphanRemoval = true)
	protected Set<PropertyBinding> getPropertyBindings() {
		return propertyBindings;
	}

	protected void setPropertyBindings(Set<PropertyBinding> propertyBindings) {
		this.propertyBindings = propertyBindings;
	}

	@ManyToOne(optional = false, fetch = FetchType.LAZY)
	public ValueType getValueType() {
		return valueType;
	}

	public void setValueType(ValueType valueType) {
		this.valueType = valueType;
	}

	@OneToMany(fetch = FetchType.EAGER, cascade = CascadeType.ALL, orphanRemoval = true)
	@JoinColumn(name = "PROPERTY_ID")
	public Set<Value> getValues() {
		return values;
	}

	public void setValues(Set<Value> values) {
		this.values = values;
	}

	public int getMaxOccurs() {
		return maxOccurs;
	}

	public void setMaxOccurs(int maxOccurs) {
		this.maxOccurs = maxOccurs;
	}

	public int getMinOccurs() {
		return minOccurs;
	}

	public void setMinOccurs(int minOccurs) {
		this.minOccurs = minOccurs;
	}

	public int getOrderIndex() {
		return orderIndex;
	}

	public void setOrderIndex(int orderIndex) {
		this.orderIndex = orderIndex;
	}

	public boolean isConfidential() {
		return confidential;
	}

	public void setConfidential(boolean confidential) {
		this.confidential = confidential;
	}

	public boolean isEncrypted() {
		return encrypted;
	}

	public void setEncrypted(boolean encrypted) {
		this.encrypted = encrypted;
	}

	public boolean isConsumerReadOnly() {
		return consumerReadOnly;
	}

	public void setConsumerReadOnly(boolean readonly) {
		this.consumerReadOnly = readonly;
	}

	public boolean isConsumerVisible() {
		return consumerVisible;
	}

	public void setConsumerVisible(boolean consumerVisible) {
		this.consumerVisible = consumerVisible;
	}

}