/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.model.custom;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import ws.ladder.db.base.Category;
import ws.ladder.db.vo.base.CategoryVO;
import ws.ladder.model.AggregateAction;
import ws.ladder.model.CallInfo;
import ws.ladder.model.ExtendedDataObject;
import ws.ladder.model.Node;
import ws.ladder.util.ClassUtil;
import ws.ladder.util.I18NUtils;
import ws.ladder.util.ObjectCreator;


public class TestAssociationStrategy extends DefaultAssociationStrategy {
	private static final Logger logger = LogManager.getLogger(new Exception().getStackTrace()[0].getClassName());

	/**
	 * The strategy used to associate with an existing object based on the source information
	 * @param callInfo
	 * @throws Exception 
	 */
	public Object execute(CallInfo callInfo, ObjectCreator oc) {
		return super.execute(callInfo, oc);
	}

	/**
	 * Method that indicates if the source object should be persisted
	 * @param callInfo
	 */
	public boolean doCreate(CallInfo callInfo) {
		boolean result = super.doCreate(callInfo);

		// special processing for Category association
		if(callInfo.getTargetProperty() != null ) {
			if(Category.class.isAssignableFrom(callInfo.getTargetProperty().getType().getInstanceClass())) {
				// We also retrieve Category objects
				if(callInfo.getSettings().getAction() == AggregateAction.READ)
					return true;

				// If the associated entity is a transient Category, we need to persist it if its type is extensible
				ExtendedDataObject target = (ExtendedDataObject) callInfo.getTarget();

				if(target == null || !((Node)target).isPersistent()) {
					boolean isExtensible = false;
					String categoryName = null;
					String categoryTypeName = null;
					if( ((ExtendedDataObject) callInfo.getSource()).isReferenceType() ) {
						Category category = (Category) ClassUtil.getInstance(callInfo.getSource()); // We have to look at the source since this is a transient object
						if(category.getCategoryType() != null && category.getCategoryType().isExtensible())
							result = true;
						categoryName = category.getName();
						categoryTypeName = (category.getCategoryType() == null) ? null : category.getCategoryType().getName();
					} else {
						CategoryVO category = (CategoryVO) ClassUtil.getInstance(callInfo.getSource()); // We have to look at the source since this is a transient object
						if(category.getCategoryType() != null && category.getCategoryType().isExtensible())
							result = true;		
						categoryName = category.getName();
						categoryTypeName = (category.getCategoryType() == null) ? null : category.getCategoryType().getName();						
					}
					if(isExtensible)
						result = true;
					else {
						String[] params = new String[2];
						params[0] = categoryName;
						params[1] = categoryTypeName;

						// Uses users resource bundle
						logger.error(I18NUtils.getResource(  "exception.cannotAddUnextensibleCategory",I18NUtils.CORE_RESOURCES, params));
					}				
				}
			} 
		}

		return result;
	}
}
