/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.model.view;

import java.util.HashMap;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import ws.ladder.model.CallInfo;

public class JPAQueryBuilder extends QueryBuilder {

	@PersistenceContext
	private EntityManager entityManager;

	@Override
	public Query createQuery(CallInfo callInfo) {
		Map<String, Object> mutableFilters = new HashMap<String, Object>(callInfo.getSettings().getFilters());
		String queryString = constructQuery(mutableFilters);
		
		Query result = null;
		if(isSQL()) {
			result = new JPAQuery(entityManager.createNativeQuery(queryString));
		} else
			result = new JPAQuery(entityManager.createQuery(queryString));

		for(String parameterName: mutableFilters.keySet())
			result.setParameter(parameterName, mutableFilters.get(parameterName));

		if(doAddIdentifier() && result.hasParameter(ID_PARAMETER_NAME))
			result.setParameter(ID_PARAMETER_NAME, getEntity().getIdentifierValue());		
		
		return result;		
	}

}
