/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.model.view.expression;

import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import ws.ladder.model.view.ViewProperty;
import ws.ladder.util.ClassUtil;

public abstract class AbstractFunctionExpression implements Expression {

	protected String expression;
	protected String attributeName;
	protected String parameterName;
	protected boolean orderBy;
	protected String normalizedAttributeName;

	public String getNormalizedAttributeName() {
		return normalizedAttributeName;
	}

	public void setNormalizedAttributeName(String normalizedAttributeName) {
		this.normalizedAttributeName = normalizedAttributeName;
	}
	
	public boolean isOrderBy() {
		return false;
	}

	public String getAttributeName() {
		return attributeName;
	}

	public void setAttributeName(String attributeName) {
		this.attributeName = attributeName;
	}

	public String getParameterName() {
		return parameterName;
	}

	public void setParameterName(String parameterName) {
		this.parameterName = parameterName;
	}

	protected abstract String getAttributePattern();
	public abstract String getQueryString();

	public String getExpression() {
		return expression;
	}

	public void setExpression(String value) {
		this.expression = value;
	}

	public AbstractFunctionExpression copy() {
		AbstractFunctionExpression result;
		try {
			result = this.getClass().newInstance();
		} catch (Exception e) {
			throw ClassUtil.wrapRun(e);
		}
		result.setExpression(this.getExpression());

		return result;
	}

	public void init() {
		// An enhancement would be to return a set of attributes if an expression consists of multiple filter functions			
		Pattern pattern = Pattern.compile( getAttributePattern() ); 
		Matcher matcher = pattern.matcher(getExpression());
		matcher.find();

		if(matcher.matches()) {
			if(matcher.groupCount() > 1)
				attributeName = matcher.group(2);
			if(matcher.groupCount() > 2)
				parameterName = matcher.group(3);
		} 
	}

	public void normalize(Map<String, String> normalizedNames) {	
		normalizedAttributeName = normalizedNames.get( ViewProperty.qualifyProperty(attributeName) );
	}

	public Object getNormalizedValue(Object object) {
		return object;
	}	
}
