/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.service;

import java.util.Map;
import java.util.UUID;

import ws.ladder.model.ExtendedProperty;
import ws.ladder.model.Type;
import ws.ladder.util.ClassUtil;

import com.basho.riak.client.RiakFactory;
import com.basho.riak.client.http.RiakClient;
import com.basho.riak.client.http.RiakObject;
import com.basho.riak.client.http.response.BucketResponse;

public class RiakPersistenceOrchestrator extends AbstractPersistenceOrchestrator {

	RiakClient riakClient;   
	
	protected RiakClient getRiakClient() {
		try {
		if(riakClient == null)
			riakClient = (RiakClient) RiakFactory.httpClient();
		} catch(Exception e) {
			throw ClassUtil.wrapRun(e);
		}
		
		return riakClient;
	}

	/**
	 * Save the entity in the persistence store
	 * 
	 * @param entity
	 * @return
	 */
	@Override
	public void saveOrUpdate(Object entity) {
		// TODO: allow riak to generate id
		RiakObject ro = new RiakObject(getRiakClient(), ClassUtil.getBucketName(entity.getClass()), UUID.randomUUID().toString());
		getRiakClient().store(ro);
		// TODO: Get the vClock information and store in the EDOI version attribute
	}

	/**
	 * Delete the entity from the persistence store
	 * 
	 * @param entity
	 */
	@Override
	public void delete(Object entity) {
		// TODO: get the identifier value
		getRiakClient().delete(ClassUtil.getBucketName(entity.getClass()), "asdfasdfasd");		
	}

	@Override
	public void flush() {
		
	}

	@Override
	public Object findById(Class<?> persistentClass, Object id) {
	   return getRiakClient().fetch(ClassUtil.getBucketName(persistentClass), id.toString());
	}
	

	@Override
	public Object findByProperty(Type type, Map<String, String> propertyValues) {
		Object result = null; 
		Class<?> persistentClass = type.getInstanceClass();
		
		// Need to use the query API
		BucketResponse br = getRiakClient().listBucket(ClassUtil.getBucketName(persistentClass));
		resultList: for(String key: br.getBucketInfo().getKeys()) {
			Object obj = getRiakClient().fetch(ClassUtil.getBucketName(persistentClass), key);
			for(String propertyName: propertyValues.keySet()) {
				ExtendedProperty property = (ExtendedProperty) type.getProperty(propertyName);
				if(!property.getValue(obj).equals(propertyValues.get(property)))
					continue resultList;
			}
			
			result = obj; // break;
		}
		
		return result;
	}		

	@Override
	public boolean supportsVersionTracking() {
		return false;
	}

	@Override
	public QueryCapability getQueryCapability() {
		return new EmptyQueryCapability();
	}

	@Override
	public boolean needsExplicitSave() {
		return false;
	}
}